@@ -76,6 +76,8 @@ static void photdistort(casu_fits *im, cpl_table *outtab);
 static void getproj_pixsize(cpl_wcs *wcs, double crv1, double tand, 
                             double secd, double x, double y, double *pixsize);
 
+static float propertylist_get_float_or_default(const cpl_propertylist * list,
+		const char * name, const float default_val);
 
 /**@{*/
 
@@ -201,8 +203,8 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
     float **stdmagptr,*resall3,*resall5,apcor3,apcor5,exptime,**magerrptr;
     float airmass,*catcore3,*catcore5,*resim3,*resim5,cf,fluxmag3,fluxmag5;
     float refmag,extinct,dm3,dm5,med3,mad,med5,sig3,sig5,blankvalue2=0.0;
-    float rcore,lim3,lim5,dx,skylev,skbrt,*ebmvall,*zps3,*zps5,medebv,med;
-    float sig3det,sig5det,*ebmv,blankvalue=-99.0,satlev,satmag,sumlim,sumskbr;
+    float rcore,lim5,dx,skylev,skbrt,*ebmvall,*zps3,*zps5,medebv,med;
+    float sig3det,sig5det,*ebmv,blankvalue=-99.0,satlev,sumskbr;
     float sknoise,sumsat;
     int nresall,nalloc_resall,i,j,k,ncat,nresim,nresimtot;
     int ngood,oops,nlim,isrubbish,*dodgy,crud;
@@ -276,7 +278,7 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
        variables. */
 
     nlim = 0;
-    sumlim = 0.0;
+    float sumlim = 0.0;
     sumsat = 0.0;
     sumskbr = 0.0;
     for (i = 0; i < nimages; i++) {
@@ -293,12 +295,6 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
             write_hdr_1(ehu_im,0,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1);
             write_hdr_1(ehu_cat,0,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0);
             dodgy[i] = 1;
-            /* cpl_propertylist_update_double(ehu_im,"ABMAGSAT",0.0); */
-            /* cpl_propertylist_set_comment(ehu_im,"ABMAGSAT", */
-            /*                              "Saturation limit for point sources"); */
-            /* cpl_propertylist_update_double(ehu_cat,"ABMAGSAT",0.0); */
-            /* cpl_propertylist_set_comment(ehu_cat,"ABMAGSAT", */
-            /*                              "Saturation limit for point sources"); */
             continue;
         }           
 
@@ -313,12 +309,6 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
             write_hdr_1(ehu_im,ncat,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1);
             write_hdr_1(ehu_cat,ncat,0.0,1.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0);
             dodgy[i] = 1;
-            /* cpl_propertylist_update_double(ehu_im,"ABMAGSAT",0.0); */
-            /* cpl_propertylist_set_comment(ehu_im,"ABMAGSAT", */
-            /*                              "Saturation limit for point sources"); */
-            /* cpl_propertylist_update_double(ehu_cat,"ABMAGSAT",0.0); */
-            /* cpl_propertylist_set_comment(ehu_cat,"ABMAGSAT", */
-            /*                              "Saturation limit for point sources"); */
             continue;
         }
 
@@ -453,17 +443,17 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         freespace(resim5);
 
         /* Calculate the limiting magnitudes */
-
+        float lim3 = 0;
         if (! crud) {
-            lim3 = med3 - 2.5*log10((5.0*sknoise*rcore*sqrt(CPL_MATH_PI))/exptime) -
-                apcor3 - extinct;
+            lim3 = casu_calculate_abmag_lim(med3 + 2.5 * log10(exptime) - extinct,
+            		sknoise, rcore, exptime, apcor3, extinct);
             lim5 = med5 - 2.5*log10((5.0*sknoise*2.0*rcore*sqrt(CPL_MATH_PI))/exptime) -
                 apcor5 - extinct;
         } else {
             med3 = p.default_zp;
             med5 = p.default_zp;
-            lim3 = med3 - 2.5*log10((5.0*sknoise*rcore*sqrt(CPL_MATH_PI))/exptime) -
-                apcor3 - extinct;
+            lim3 = casu_calculate_abmag_lim(med3 + 2.5 * log10(exptime) - extinct,
+            		sknoise, rcore, exptime, apcor3, extinct);
             lim5 = med5 - 2.5*log10((5.0*sknoise*2.0*rcore*sqrt(CPL_MATH_PI))/exptime) -
                 apcor5 - extinct;
         }
@@ -473,8 +463,14 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         skbrt = med3 - 2.5*log10(skylev/(exptime*dx*dx)) - extinct;
 
         /* Calculate the saturation magnitude */
+        const float psf_fwhm =  propertylist_get_float_or_default(ehu_cat,"PSF_FWHM", 1.0f);
+        const float mean_sky =  propertylist_get_float_or_default(ehu_cat,"ESO QC MEAN_SKY", 1.0f);
+        /*Using image header because HAWKI does not have pixelscale in catalog*/
+        const float pixel_scale = propertylist_get_float_or_default(ehu_im,"ESO QC WCS_SCALE", 1.0f);
+
+        const float satmag = casu_calculate_abmag_sat(med3 + 2.5 * log10(exptime) - extinct,
+        		65e3f, mean_sky, psf_fwhm, pixel_scale, exptime);
 
-        satmag = med3 - 2.5*log10(satlev/exptime) - extinct;
         if (!crud) {
             sumlim += lim3;
             sumsat += satmag;
@@ -595,9 +591,6 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         cpl_propertylist_update_double(ehu_im,"ESO QC MAGZERR",sig3det);
         cpl_propertylist_set_comment(ehu_im,"ESO QC MAGZERR",
                                      "[mag] photometric zeropoint error");
-        cpl_propertylist_update_double(ehu_im,"ESO QC LIMITING_MAG",sumlim);
-        cpl_propertylist_set_comment(ehu_im,"ESO QC LIMITING_MAG",
-                                     "[mag] 5 sigma limiting mag.");
         cpl_propertylist_update_double(ehu_im,"ESO QC SKYBRIGHT",sumskbr);
         cpl_propertylist_set_comment(ehu_im,"ESO QC SKYBRIGHT",
                                      "[mag/arcsec**2] sky brightness");
@@ -618,8 +611,11 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
             cpl_propertylist_set_comment(ehu_im,"ABMAGSAT",
                                          "Saturation limit for point sources");
             cpl_propertylist_update_double(ehu_im,"ABMAGLIM",sumlim);
-            cpl_propertylist_set_comment(ehu_im,"ABMAGSAT",
-                                         "Saturation limit for point sources");
+            cpl_propertylist_set_comment(ehu_im,"ABMAGLIM",
+                                         "[mag] 5 sigma limiting mag");
+            cpl_propertylist_update_double(ehu_im,"ESO QC LIMITING_MAG",sumlim);
+            cpl_propertylist_set_comment(ehu_im,"ESO QC LIMITING_MAG",
+                                         "[mag] 5 sigma limiting mag.");
         }
         
         /* Now the catalogues */
@@ -654,9 +650,6 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
         cpl_propertylist_update_double(ehu_cat,"ESO QC MAGZERR",sig3det);
         cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGZERR",
                                      "[mag] photometric zeropoint error");
-        cpl_propertylist_update_double(ehu_cat,"ESO QC LIMITING_MAG",sumlim);
-        cpl_propertylist_set_comment(ehu_cat,"ESO QC LIMITING_MAG",
-                                     "[mag] 5 sigma limiting mag.");
         cpl_propertylist_update_double(ehu_cat,"ESO QC SKYBRIGHT",sumskbr);
         cpl_propertylist_set_comment(ehu_cat,"ESO QC SKYBRIGHT",
                                      "[mag/arcsec**2] sky brightness");
@@ -677,8 +670,11 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
             cpl_propertylist_set_comment(ehu_cat,"ABMAGSAT",
                                          "Saturation limit for point sources");
             cpl_propertylist_update_double(ehu_cat,"ABMAGLIM",sumlim);
-            cpl_propertylist_set_comment(ehu_cat,"ABMAGSAT",
-                                         "Saturation limit for point sources");
+            cpl_propertylist_set_comment(ehu_cat,"ABMAGLIM",
+                                         "[mag] 5 sigma limiting mag");
+            cpl_propertylist_update_double(ehu_cat,"ESO QC LIMITING_MAG",sumlim);
+            cpl_propertylist_set_comment(ehu_cat,"ESO QC LIMITING_MAG",
+                                         "[mag] 5 sigma limiting mag.");
         }
 
         /* Update the matched standards headers */
@@ -715,9 +711,6 @@ extern int casu_photcal_extinct(casu_fits **images, casu_tfits **mstds,
 	    cpl_propertylist_update_double(ehu_cat,"ESO QC MAGZERR",sig3det);
 	    cpl_propertylist_set_comment(ehu_cat,"ESO QC MAGZERR",
 					 "[mag] photometric zeropoint error");
-	    cpl_propertylist_update_double(ehu_cat,"ESO QC LIMITING_MAG",sumlim);
-	    cpl_propertylist_set_comment(ehu_cat,"ESO QC LIMITING_MAG",
-					 "[mag] 5 sigma limiting mag.");
 	    cpl_propertylist_update_double(ehu_cat,"ESO QC SKYBRIGHT",sumskbr);
 	    cpl_propertylist_set_comment(ehu_cat,"ESO QC SKYBRIGHT",
 					 "[mag/arcsec**2] sky brightness");
@@ -1090,9 +1083,6 @@ static void write_hdr_1(cpl_propertylist *pl, int nresim, float med3,
         cpl_propertylist_update_int(pl,"ESO QC MAGNZPT",nresim);
         cpl_propertylist_set_comment(pl,"ESO QC MAGNZPT",
                                      "number of stars in magzpt calc");
-        cpl_propertylist_update_double(pl,"ESO QC LIMITING_MAG",lim3);
-        cpl_propertylist_set_comment(pl,"ESO QC LIMITING_MAG",
-                                     "[mag] 5 sigma limiting mag.");
         cpl_propertylist_update_double(pl,"ESO QC SKYBRIGHT",skybrt);
         cpl_propertylist_set_comment(pl,"ESO QC SKYBRIGHT",
                                      "[mag/arcsec**2] sky brightness");
@@ -1115,7 +1105,10 @@ static void write_hdr_1(cpl_propertylist *pl, int nresim, float med3,
                                  "[mag] 5 sigma limiting mag 1*rcore.");
     cpl_propertylist_update_double(pl,"ABMAGLIM",lim3);
     cpl_propertylist_set_comment(pl,"ABMAGLIM",
-                                 "[mag] 5 sigma limiting mag 1*rcore.");
+                                 "[mag] 5 sigma limiting mag");
+    cpl_propertylist_update_double(pl,"ESO QC LIMITING_MAG",lim3);
+    cpl_propertylist_set_comment(pl,"ESO QC LIMITING_MAG",
+                                 "[mag] 5 sigma limiting mag.");
     cpl_propertylist_update_double(pl,"ESO DRS ZPIM2",med5);
     cpl_propertylist_set_comment(pl,"ESO DRS ZPIM2",
                                  "[mag] zeropoint 2*rcore this image only");
@@ -1314,6 +1307,42 @@ static void photdistort(casu_fits *im, cpl_table *outtab) {
     freewcs(wcs);
 }
 
+/*---------------------------------------------------------------------------*/
+/**
+    \par Name:
+        propertylist_get_float_or_default
+    \par Purpose:
+        Get the property of the default value
+    \par Description:
+        Get the property of the default value
+    \par Language:
+        C
+    \param list
+        The input propertylist
+    \param name
+        Tag name
+    \param default_val
+        Default value to be returned if the propertylist has no value
+	\retval the corresponding value in the propertylist or default if not
+			available.
+	\author
+        Matteo Salmistraro, ESO
+*/
+/*---------------------------------------------------------------------------*/
+static float propertylist_get_float_or_default(const cpl_propertylist * list, const char * name,
+		const float default_val){
+
+	cpl_boolean value_missing_or_mismatch = !cpl_propertylist_has(list, name);
+	value_missing_or_mismatch = value_missing_or_mismatch || (cpl_propertylist_get_type(list, name) != CPL_TYPE_DOUBLE
+			&& cpl_propertylist_get_type(list, name) != CPL_TYPE_FLOAT);
+	if(value_missing_or_mismatch) {
+		cpl_msg_warning(cpl_func, "Unable to extract %s, fallback to %f", name, default_val);
+		return default_val;
+	}
+
+	return cpl_propertylist_get_float(list, name);
+}
+
 /*---------------------------------------------------------------------------*/
 /**
     \par Name:
@@ -1578,6 +1607,46 @@ static void radectolb(double ra, double dec, float epoch, double *gal_l,
     *gal_b = asin(sin_b);
 }
 
+/*---------------------------------------------------------------------------*/
+/**
+    \par Name:
+        casu_calculate_abmag_lim
+    \par Purpose:
+        Calculate ABMAGLIM
+    \par Description:
+        Calculate ABMAGLIM
+    \par Language:
+        C
+    \param zeropoint
+        The zeropoint
+    \param skynoise
+        The sky noise
+    \param r_core
+        The radius of the core
+    \param eff_expt
+        The effective exposure time
+    \param apcor3
+        The aperture correction
+	\param extinction
+        The extintion
+    \author
+        Matteo Salmistraro, ESO
+*/
+/*---------------------------------------------------------------------------*/
+extern float
+casu_calculate_abmag_lim(const float zeropoint, const float skynoise,
+		const float r_core, const float eff_expt, const float apcor3, const float extinction){
+	return zeropoint - 2.5f * log10f(5.0f * skynoise * r_core * sqrt(CPL_MATH_PI) / eff_expt) - apcor3 - extinction;
+}
+
+extern float
+casu_calculate_abmag_sat(const float zeropoint, const float satlev,
+		const float mean_sky, const float psf_fwhm, const float pixel_scale,
+		const float exptime){
+	const float val = (CPL_MATH_PI_4 * CPL_MATH_LN2) * (satlev - mean_sky) * powf((psf_fwhm / pixel_scale), 2.0) / exptime;
+	return	zeropoint - 2.5f*log10f(val);
+}
+
 /**@}*/
 
 /*
