@@ -1,4 +1,4 @@
-/* $Id: irplib_stdstar.c,v 1.41 2012/05/04 13:58:12 cgarcia Exp $
+/* $Id: irplib_stdstar.c,v 1.45 2013-03-01 10:27:07 llundin Exp $
  *
  * This file is part of the irplib package
  * Copyright (C) 2002,2003 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: cgarcia $
- * $Date: 2012/05/04 13:58:12 $
- * $Revision: 1.41 $
- * $Name: hawki-1_8_12 $
+ * $Author: llundin $
+ * $Date: 2013-03-01 10:27:07 $
+ * $Revision: 1.45 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,15 +33,25 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
+#include "irplib_stdstar.h"
+#include "irplib_utils.h"
+#include "irplib_wcs.h"
+#include <cpl.h>
+
 #include <string.h>
 #include <math.h>
 #include <float.h>
 
-#include <cpl.h>
+/*-----------------------------------------------------------------------------
+                                   Defines
+ -----------------------------------------------------------------------------*/
 
-#include "irplib_stdstar.h"
-#include "irplib_utils.h"
-#include "irplib_wcs.h"
+#if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(6, 3, 0)
+#define IRPLIB_FRAMESET_GET_CONST cpl_frameset_get_position_const
+#else
+/* Remove this branch once CPL versions less than 6.3 are no longer supported */
+#define IRPLIB_FRAMESET_GET_CONST cpl_frameset_get_frame_const
+#endif
 
 
 /*----------------------------------------------------------------------------*/
@@ -58,7 +68,7 @@
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Write the ASCII catalogs as FITS files
-  @param    set_in          Input frameset wher the product is registered
+  @param    set_in          Input frameset where the product is registered
   @param    set_raw         Set of ASCII catalogs
   @param    recipe_name     Recipe name
   @param    pro_cat         PRO.CATG
@@ -66,7 +76,7 @@
   @param    package_name    Usually PACKAGE "/" PACKAGE_VERSION
   @param    ins_name        Instrument name
   @param    convert_ascii_table Conversion function
-  @return   0 if ok, -1 in error case
+  @return   CPL_ERROR_NONE if ok or else the relevant CPL error code
 
   Every catalog will be written in a different extension, where EXTNAME
   is the name of the catalog ASCII file name.
@@ -75,98 +85,97 @@
   should be defined in each instrument
  */
 /*----------------------------------------------------------------------------*/
-int irplib_stdstar_write_catalogs(
-        cpl_frameset        *   set_in,
-        const cpl_frameset  *   set_raw,
-        const char          *   recipe_name,
-        const char          *   pro_cat,
-        const char          *   pro_type,
-        const char          *   package_name,
-        const char          *   ins_name,
-        cpl_table * (*convert_ascii_table)(const char *))
+cpl_error_code
+irplib_stdstar_write_catalogs(cpl_frameset        *   set_in,
+                              const cpl_frameset  *   set_raw,
+                              const char          *   recipe_name,
+                              const char          *   pro_cat,
+                              const char          *   pro_type,
+                              const char          *   package_name,
+                              const char          *   ins_name,
+                              cpl_table * (*convert_ascii_table)(const char *))
 {
-    cpl_table           *   out ;
-    cpl_propertylist    *   plist ;
-    cpl_propertylist    *   plist_ext ;
-    cpl_parameterlist   *   parlist ;
-    const char          *   cat_name ;
-    char                *   out_name ;
-    int                     nb_catalogs ;
-    const cpl_frame     *   cur_frame ;
-    int                     i ;
+    /* Number of catalogs */
+    const cpl_size     nb_catalogs = cpl_frameset_get_size(set_raw);
+    cpl_propertylist * plist_ext;
+    char             * out_name;
+    cpl_error_code     error = CPL_ERROR_NONE;
+    cpl_size           i;
 
     /* Check entries */
-    if (set_in == NULL) return CPL_ERROR_NULL_INPUT;
-    if (set_raw == NULL) return CPL_ERROR_NULL_INPUT;
-    if (recipe_name == NULL) return CPL_ERROR_NULL_INPUT;
-    if (pro_cat == NULL) return CPL_ERROR_NULL_INPUT;
-    if (ins_name == NULL) return CPL_ERROR_NULL_INPUT;
+    if (set_in == NULL) return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+    if (set_raw == NULL) return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+    if (recipe_name == NULL) return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+    if (pro_cat == NULL) return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+    if (ins_name == NULL) return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+    if (convert_ascii_table == NULL) return
+        cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
 
     /* Define the file name */
-    out_name = cpl_sprintf("%s.fits", recipe_name) ;
+    out_name = cpl_sprintf("%s" CPL_DFS_FITS, recipe_name);
     
-    /* Number of catalogs */
-    nb_catalogs = cpl_frameset_get_size(set_raw) ;
-
-    /* Get the catalog name */
-    cur_frame = cpl_frameset_get_frame_const(set_raw, 0) ;
-    cat_name = cpl_frame_get_filename(cur_frame) ;
+    plist_ext = cpl_propertylist_new();
     
-    /* Create the output table */
-    if ((out = convert_ascii_table(cat_name)) == NULL) {
-        cpl_free(out_name) ;
-        return CPL_ERROR_UNSPECIFIED;
-    }
-
-    /* Some keywords  */
-    plist = cpl_propertylist_new() ;
-    cpl_propertylist_append_string(plist, "INSTRUME", ins_name) ;
-    cpl_propertylist_append_string(plist, CPL_DFS_PRO_CATG, pro_cat) ;
-    if (pro_type != NULL) {
-        cpl_propertylist_append_string(plist, CPL_DFS_PRO_TYPE, pro_type) ;
-    }
-    plist_ext = cpl_propertylist_new() ;
-    cpl_propertylist_append_string(plist_ext, "EXTNAME", cat_name) ;
-
-    /* Write the table */
-    parlist = cpl_parameterlist_new() ;
-    cpl_dfs_save_table(set_in, 
-            NULL,
-            parlist,
-            set_raw,
-            NULL,
-            out,
-            plist_ext,
-            recipe_name,
-            plist,
-            NULL,
-            package_name,
-            out_name) ;
-    cpl_parameterlist_delete(parlist) ;
-    cpl_propertylist_delete(plist) ;
-    cpl_propertylist_delete(plist_ext) ;
-    cpl_table_delete(out) ;
-    
-    /* Append the next catalogs */
-    for (i=1 ; i<nb_catalogs ; i++) {
+    /* Process the catalogs */
+    for (i = 0; i < nb_catalogs; i++) {
         /* Get the catalog name */
-        cur_frame = cpl_frameset_get_frame_const(set_raw, i) ;
-        cat_name = cpl_frame_get_filename(cur_frame) ;
+        const cpl_frame * cur_frame = IRPLIB_FRAMESET_GET_CONST(set_raw, i);
+        const char      * cat_name = cpl_frame_get_filename(cur_frame);
+
+        cpl_table       * out = convert_ascii_table(cat_name);
      
         /* Create the output table */
-        if ((out = convert_ascii_table(cat_name)) == NULL) {
-            cpl_free(out_name) ;
-            return CPL_ERROR_UNSPECIFIED;
+        if (out == NULL) {
+            error = cpl_error_get_code() ? cpl_error_set_where(cpl_func)
+                : cpl_error_set(cpl_func, CPL_ERROR_UNSPECIFIED);
+            break;
         }
 
-        plist_ext = cpl_propertylist_new() ;
-        cpl_propertylist_append_string(plist_ext, "EXTNAME", cat_name) ;
-        cpl_table_save(out, NULL, plist_ext, out_name, CPL_IO_EXTEND) ;
-        cpl_table_delete(out) ;
-        cpl_propertylist_delete(plist_ext) ;
+        if (cpl_table_get_nrow(out) == 0) {
+            cpl_table_delete(out);
+            error = cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
+                                          "Empty catalogue %d in '%s'",
+                                          (int)i+1, cat_name);
+            break;
+        }
+
+        cpl_propertylist_update_string(plist_ext, "EXTNAME", cat_name);
+
+        /* Write the table */
+        if (i == 0) {
+            cpl_parameterlist * parlist = cpl_parameterlist_new();
+            cpl_propertylist  * plist   = cpl_propertylist_new();
+
+            /* Mandatory keywords */
+            cpl_propertylist_append_string(plist, "INSTRUME", ins_name);
+            cpl_propertylist_append_string(plist, CPL_DFS_PRO_CATG, pro_cat);
+            if (pro_type != NULL) {
+                cpl_propertylist_append_string(plist, CPL_DFS_PRO_TYPE,
+                                               pro_type);
+            }
+
+            error = cpl_dfs_save_table(set_in, NULL, parlist, set_raw, NULL,
+                                       out, plist_ext, recipe_name, plist,
+                                       NULL, package_name, out_name);
+            cpl_parameterlist_delete(parlist);
+            cpl_propertylist_delete(plist);
+        } else {
+            error = cpl_table_save(out, NULL, plist_ext, out_name,
+                                   CPL_IO_EXTEND);
+        }
+
+        cpl_table_delete(out);
+
+        if (error) {
+            (void)cpl_error_set_where(cpl_func);
+            break;
+        }
     }
-    cpl_free(out_name) ;
-    return 0 ;
+
+    cpl_propertylist_delete(plist_ext);
+    cpl_free(out_name);
+
+    return error;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -187,75 +196,76 @@ cpl_table * irplib_stdstar_load_catalog(
         const char  *   filename, 
         const char  *   ext_name)
 {
-    int                     next ;
-    cpl_propertylist    *   plist ;
-    const char          *   cur_name ;
-    cpl_table           *   out ;
-    cpl_table           *   out_cur ;
-    cpl_frame           *   cur_frame ;
-    int                     i ;
+    int                     next;
+    cpl_table           *   out;
+    cpl_table           *   out_cur;
+    cpl_frame           *   cur_frame;
+    int                     i;
 
     /* Check entries */
-    if (filename == NULL) return NULL ;
-    if (ext_name == NULL) return NULL ;
+    if (filename == NULL) return NULL;
+    if (ext_name == NULL) return NULL;
     
     /* Initialise */
-    out = NULL ;
+    out = NULL;
     
     /* Get the number of extensions in the catalog */
-    cur_frame = cpl_frame_new() ;
-    cpl_frame_set_filename(cur_frame, filename) ;
-    next = cpl_frame_get_nextensions(cur_frame) ;
-    cpl_frame_delete(cur_frame) ;
+    cur_frame = cpl_frame_new();
+    cpl_frame_set_filename(cur_frame, filename);
+    next = cpl_frame_get_nextensions(cur_frame);
+    cpl_frame_delete(cur_frame);
 
     /* Loop on the extentions */
-    for (i=0 ; i<next ; i++) {
+    for (i=0; i<next; i++) {
+        cpl_propertylist    *   plist;
+        const char          *   cur_name;
+
         /* Check the name of the current extension */
         if ((plist = cpl_propertylist_load_regexp(filename, i+1, "EXTNAME", 
                         0)) == NULL) {
             cpl_msg_error(cpl_func, "Cannot load header of %d th extension",
                     i+1);
-            return NULL ;
+            return NULL;
         }
-        cur_name = cpl_propertylist_get_string(plist, "EXTNAME") ;
+        cur_name = cpl_propertylist_get_string(plist, "EXTNAME");
         
         /* Check the current extension */
         if (!strcmp(cur_name, ext_name)) {
             /* Load the table */
             if (out == NULL) {
-                out = cpl_table_load(filename, i+1, 1) ;
+                out = cpl_table_load(filename, i+1, 1);
                 cpl_table_new_column(out, IRPLIB_STDSTAR_CAT_COL, CPL_TYPE_STRING);
                 cpl_table_fill_column_window_string(out, IRPLIB_STDSTAR_CAT_COL,
                                                     0, cpl_table_get_nrow(out),
                                                     cur_name);
                 if (out == NULL) {
-                    cpl_msg_error(cpl_func, "Cannot load extension %d", i+1) ;
-                    cpl_propertylist_delete(plist) ;
-                    return NULL ;
+                    cpl_msg_error(cpl_func, "Cannot load extension %d", i+1);
+                    cpl_propertylist_delete(plist);
+                    return NULL;
                 }
             }
         } else if (!strcmp(ext_name, "all")) {
             /* Load the table and append it */
             if (i==0) {
                 /* Load the first table */
-                out = cpl_table_load(filename, i+1, 1) ;
+                out = cpl_table_load(filename, i+1, 1);
                 cpl_table_new_column(out, IRPLIB_STDSTAR_CAT_COL, CPL_TYPE_STRING);
                 cpl_table_fill_column_window_string(out, IRPLIB_STDSTAR_CAT_COL,
                                                     0, cpl_table_get_nrow(out),
                                                     cur_name);
                 if (out == NULL) {
-                    cpl_msg_error(cpl_func, "Cannot load extension %d", i+1) ;
-                    cpl_propertylist_delete(plist) ; 
-                    return NULL ;
+                    cpl_msg_error(cpl_func, "Cannot load extension %d", i+1);
+                    cpl_propertylist_delete(plist); 
+                    return NULL;
                 }
             } else {
                 /* Load the current table */
-                out_cur = cpl_table_load(filename, i+1, 1) ;
+                out_cur = cpl_table_load(filename, i+1, 1);
                 if (out_cur == NULL) {
-                    cpl_msg_error(cpl_func, "Cannot load extension %d", i+1) ;
-                    cpl_table_delete(out) ;
-                    cpl_propertylist_delete(plist) ; 
-                    return NULL ;
+                    cpl_msg_error(cpl_func, "Cannot load extension %d", i+1);
+                    cpl_table_delete(out);
+                    cpl_propertylist_delete(plist); 
+                    return NULL;
                 }
                 cpl_table_new_column(out_cur, IRPLIB_STDSTAR_CAT_COL, CPL_TYPE_STRING);
                 cpl_table_fill_column_window_string(out_cur, IRPLIB_STDSTAR_CAT_COL,
@@ -264,18 +274,18 @@ cpl_table * irplib_stdstar_load_catalog(
                 /* Append the table */
                 if (cpl_table_insert(out, out_cur, 
                             cpl_table_get_nrow(out)) != CPL_ERROR_NONE) {
-                    cpl_msg_error(cpl_func, "Cannot merge table %d", i+1) ;
-                    cpl_table_delete(out) ;
-                    cpl_table_delete(out_cur) ;
-                    cpl_propertylist_delete(plist) ; 
-                    return NULL ;
+                    cpl_msg_error(cpl_func, "Cannot merge table %d", i+1);
+                    cpl_table_delete(out);
+                    cpl_table_delete(out_cur);
+                    cpl_propertylist_delete(plist); 
+                    return NULL;
                 }
-                cpl_table_delete(out_cur) ;
+                cpl_table_delete(out_cur);
             }
         }
-        cpl_propertylist_delete(plist) ;
+        cpl_propertylist_delete(plist);
     }
-    return out ;
+    return out;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -325,15 +335,14 @@ cpl_error_code irplib_stdstar_check_columns_exist(
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    Select the stars that are within a given distance
-  @param    cat     the catalog
-  @param    ra      RA pos
-  @param    dec     DEC pos
-  @param    dist    the distancw
-  @return   0 if ok, -1 in error case
+  @brief  Deselect the stars that are beyond a given distance
+  @param   cat   The standard star catalog, with too distant stars to unselect
+  @param   ra    Right ascension [degrees]
+  @param   dec   Declination [degrees]
+  @param   dist  The distance [degrees]
+  @return  0 if ok, -1 in error case
 
-  The stars that are within not further than dist from ra,dec are
-  selected in the table.
+  The stars that are further than dist from ra,dec are unselected in the table.
  */
 /*----------------------------------------------------------------------------*/
 int irplib_stdstar_select_stars_dist(
@@ -342,34 +351,37 @@ int irplib_stdstar_select_stars_dist(
         double          dec, 
         double          dist)
 {
-    double              distance ;
-    int                 nrows ;
-    int                 i ;
+    cpl_size            i, nrows;
     
     /* Check entries */
     if (cat == NULL) return -1;
 
     /* Get the number of selected rows */
-    nrows = cpl_table_get_nrow(cat) ;
-    
+    nrows = cpl_table_get_nrow(cat);
+
     /* Check if the columns are there */
     if (!cpl_table_has_column(cat, IRPLIB_STDSTAR_RA_COL)) {
-        cpl_msg_error(cpl_func, "Missing %s column", IRPLIB_STDSTAR_RA_COL) ;
-        return -1 ;
+        cpl_msg_error(cpl_func, "Missing column: " IRPLIB_STDSTAR_RA_COL);
+        return -1;
     }
     if (!cpl_table_has_column(cat, IRPLIB_STDSTAR_DEC_COL)) {
-        cpl_msg_error(cpl_func, "Missing %s column", IRPLIB_STDSTAR_DEC_COL) ;
-        return -1 ;
+        cpl_msg_error(cpl_func, "Missing column: " IRPLIB_STDSTAR_DEC_COL);
+        return -1;
+    }
+    
+    if (cpl_table_count_selected(cat) == 0) {
+        cpl_msg_error(cpl_func, "All %d row(s) already deselected", (int)nrows);
+        return -1;
     }
     
     /* Compute distances of the selected rows */
-    for (i=0 ; i<nrows ; i++) {
+    for (i = 0; i < nrows; i++) {
         if (cpl_table_is_selected(cat, i)) {
             /* The row is selected - compute the distance */
-            distance = irplib_wcs_great_circle_dist(ra, dec, 
+            const double distance = irplib_wcs_great_circle_dist(ra, dec, 
                     cpl_table_get_double(cat, IRPLIB_STDSTAR_RA_COL, i, NULL),
                     cpl_table_get_double(cat, IRPLIB_STDSTAR_DEC_COL, i, NULL));            
-            if (distance > dist) cpl_table_unselect_row(cat, i) ;
+            if (distance > dist) cpl_table_unselect_row(cat, i);
         }
     }
     return 0;
@@ -390,24 +402,24 @@ int irplib_stdstar_select_stars_mag(
         const char  *   mag_colname)
 {
     /* Check entries */
-    if (cat == NULL) return -1 ;
-    if (mag_colname == NULL) return -1 ;
+    if (cat == NULL) return -1;
+    if (mag_colname == NULL) return -1;
 
     /* Check that the table has the mag column */
     if (!cpl_table_has_column(cat, mag_colname)) {
         cpl_msg_error(cpl_func, "Column %s does not exist in the catalog",
-                mag_colname) ;
-        return -1 ;
+                mag_colname);
+        return -1;
     }
 
     /* Apply the selection */
     if (cpl_table_and_selected_double(cat, mag_colname, CPL_NOT_GREATER_THAN, 
                 98.0) <= 0) {
         cpl_msg_error(cpl_func, "Column %s does not exist in the catalog",
-                mag_colname) ;
-        return -1 ;
+                mag_colname);
+        return -1;
     }
-    return 0 ;
+    return 0;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -426,45 +438,45 @@ int irplib_stdstar_find_closest(
         double                  ra, 
         double                  dec)
 {
-    double              min_dist, distance ;
-    int                 nrows ;
-    int                 ind ;
-    int                 i ;
+    double              min_dist, distance;
+    int                 nrows;
+    int                 ind;
+    int                 i;
 
     /* Check entries */
-    if (cat == NULL) return -1 ;
+    if (cat == NULL) return -1;
 
     /* Initialize */
-    min_dist = 1000.0 ;
-    ind = -1 ;
+    min_dist = 1000.0;
+    ind = -1;
 
     /* Get the number of selected rows */
-    nrows = cpl_table_get_nrow(cat) ;
+    nrows = cpl_table_get_nrow(cat);
 
     /* Check if the columns are there */
     if (!cpl_table_has_column(cat, IRPLIB_STDSTAR_RA_COL)) {
-        cpl_msg_error(cpl_func, "Missing %s column", IRPLIB_STDSTAR_RA_COL) ;
-        return -1 ;
+        cpl_msg_error(cpl_func, "Missing %s column", IRPLIB_STDSTAR_RA_COL);
+        return -1;
     }
     if (!cpl_table_has_column(cat, IRPLIB_STDSTAR_DEC_COL)) {
-        cpl_msg_error(cpl_func, "Missing %s column", IRPLIB_STDSTAR_DEC_COL) ;
-        return -1 ;
+        cpl_msg_error(cpl_func, "Missing %s column", IRPLIB_STDSTAR_DEC_COL);
+        return -1;
     }
     
     /* Compute distances of the selected rows */
-    for (i=0 ; i<nrows ; i++) {
+    for (i=0; i<nrows; i++) {
         if (cpl_table_is_selected(cat, i)) {
             /* The row is selected - compute the distance */
             distance = irplib_wcs_great_circle_dist(ra, dec,
                     cpl_table_get_double(cat, IRPLIB_STDSTAR_RA_COL, i, NULL),
                     cpl_table_get_double(cat, IRPLIB_STDSTAR_DEC_COL, i, NULL));
             if (distance <= min_dist) {
-                min_dist = distance ;
-                ind = i ;
+                min_dist = distance;
+                ind = i;
             }
         }
     }
-    return ind ;
+    return ind;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -481,7 +493,7 @@ int irplib_stdstar_find_closest(
   @param    star_ra  the star RA from the catalog (output)
   @param    star_dec the star DEC from the catalog (output)
   @param    dist_am  the distance in arc minutes
-  @return   CPL_ERROR_NONE if ok, or an appropiate error case otherwise
+  @return   CPL_ERROR_NONE if ok or else the relevant CPL error code
 
   The closest star to ra, dec with defined magnitude is search in all the 
   catalogues included in catfile.
@@ -504,9 +516,9 @@ cpl_error_code irplib_stdstar_find_star(
         double                  dist_am)
 {
     cpl_errorstate prestate = cpl_errorstate_get();
-    cpl_table   *   catal ;
-    const double    dist = dist_am / 60.0 ;
-    int             ind ;
+    cpl_table   *   catal;
+    const double    dist = dist_am / 60.0;
+    int             ind;
 
     /* Check entries */
     if (catfile == NULL) return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
@@ -528,28 +540,28 @@ cpl_error_code irplib_stdstar_find_star(
     
     /* Select stars with known magnitude */
     if (irplib_stdstar_select_stars_mag(catal, band) == -1) {
-        cpl_table_delete(catal) ;
+        cpl_table_delete(catal);
         return cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT,
                                            "Cannot select stars in that band");
     }
 
     /* Select stars within a given distance */
     if (irplib_stdstar_select_stars_dist(catal, ra, dec, dist) == -1) {
-        cpl_table_delete(catal) ;
+        cpl_table_delete(catal);
         return cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT,
                                            "Cannot select close stars");
     }
 
     /* Take the closest */
     if ((ind=irplib_stdstar_find_closest(catal, ra, dec)) < 0) {
-        cpl_table_delete(catal) ;
+        cpl_table_delete(catal);
         return cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT,
                                            "Cannot get the closest star with "
                                            "known %s magnitude",band);
     }
 
     if(mag != NULL)
-        *mag = cpl_table_get_double(catal, band, ind, NULL) ;
+        *mag = cpl_table_get_double(catal, band, ind, NULL);
 
     if(name != NULL)
     {
@@ -604,47 +616,47 @@ cpl_vector * irplib_stdstar_get_conversion(
         double                  gain,
         double                  mag)
 {
-    double                      h = 6.62e-27 ;
-    double                      c = 3e18 ;
-    const cpl_vector    *       wave ;
-    const cpl_vector    *       extr ;
-    cpl_vector          *       out ;
-    double                      factor ;
+    double                      h = 6.62e-27;
+    double                      c = 3e18;
+    const cpl_vector    *       wave;
+    const cpl_vector    *       extr;
+    cpl_vector          *       out;
+    double                      factor;
 
     /* Test entries */
-    if (spec == NULL) return NULL ;
-    if (dit <= 0.0) return NULL ;
+    if (spec == NULL) return NULL;
+    if (dit <= 0.0) return NULL;
 
     /* Get the extracted spectrum */
-    wave = cpl_bivector_get_x_const(spec) ;
-    extr = cpl_bivector_get_y_const(spec) ;
+    wave = cpl_bivector_get_x_const(spec);
+    extr = cpl_bivector_get_y_const(spec);
 
     /* Get the spectrum */
-    out = cpl_vector_duplicate(extr) ;
+    out = cpl_vector_duplicate(extr);
 
     /* Divide by DIT */
-    cpl_vector_divide_scalar(out, dit) ;
+    cpl_vector_divide_scalar(out, dit);
 
     /* Divide by the surface */
-    cpl_vector_divide_scalar(out, surface) ;
+    cpl_vector_divide_scalar(out, surface);
 
     /* Multiply by the gain */
-    cpl_vector_multiply_scalar(out, gain) ;
+    cpl_vector_multiply_scalar(out, gain);
 
     /* Multiply by the difference magnitude */
-    factor = pow(10, mag/2.5) ;
-    cpl_vector_multiply_scalar(out, factor) ;
+    factor = pow(10, mag/2.5);
+    cpl_vector_multiply_scalar(out, factor);
 
     /* Divide by the dispersion */
     factor = (cpl_vector_get(wave, cpl_vector_get_size(wave)-1) -
-            cpl_vector_get(wave, 0)) / cpl_vector_get_size(wave) ;
-    cpl_vector_divide_scalar(out, factor) ;
+            cpl_vector_get(wave, 0)) / cpl_vector_get_size(wave);
+    cpl_vector_divide_scalar(out, factor);
 
     /* Multiply by the energy of the photon */
-    cpl_vector_multiply_scalar(out, h*c) ;
-    cpl_vector_divide(out, wave) ;
+    cpl_vector_multiply_scalar(out, h*c);
+    cpl_vector_divide(out, wave);
 
-    return out ;
+    return out;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -661,91 +673,93 @@ cpl_vector * irplib_stdstar_get_mag_zero(
         const cpl_vector    *   waves,
         double                  cent_wl)
 {
-    double              wmin, wmax, wstep ;
-    int                 nb_sed ;
-    const double    *   sed_x ;
-    const double    *   sed_y ;
-    cpl_bivector    *   sed_loc ;
-    double          *   sed_loc_x ;
-    double          *   sed_loc_y ;
-    cpl_vector      *   out ;
-    cpl_bivector    *   out_biv ;
-    double              f0_jan, f0_erg, cent_val ;
-    int                 i ;
+    double              wmin, wmax, wstep;
+    int                 nb_sed;
+    const double    *   sed_x;
+    const double    *   sed_y;
+    cpl_bivector    *   sed_loc;
+    double          *   sed_loc_x;
+    double          *   sed_loc_y;
+    cpl_vector      *   out;
+    cpl_bivector    *   out_biv;
+    double              f0_jan, f0_erg, cent_val;
+    int                 i;
 
     /* Test entries */
-    if (sed == NULL) return NULL ;
-    if (waves == NULL) return NULL ;
+    if (sed == NULL) return NULL;
+    if (waves == NULL) return NULL;
 
     /* Initialise */
-    nb_sed = cpl_bivector_get_size(sed) ;
-    sed_x = cpl_bivector_get_x_data_const(sed) ;
-    sed_y = cpl_bivector_get_y_data_const(sed) ;
-    wstep = sed_x[1] - sed_x[0] ;
-    wmin = cpl_vector_get(waves, 0) ;
-    wmax = cpl_vector_get(waves, cpl_vector_get_size(waves)-1) ;
+    nb_sed = cpl_bivector_get_size(sed);
+    sed_x = cpl_bivector_get_x_data_const(sed);
+    sed_y = cpl_bivector_get_y_data_const(sed);
+    wstep = sed_x[1] - sed_x[0];
+    wmin = cpl_vector_get(waves, 0);
+    wmax = cpl_vector_get(waves, cpl_vector_get_size(waves)-1);
 
     /* Expand sed with 0 to have it bigger than the required wavelengths */
-    sed_loc = cpl_bivector_new(nb_sed + 4) ;
-    sed_loc_x = cpl_bivector_get_x_data(sed_loc) ;
-    sed_loc_y = cpl_bivector_get_y_data(sed_loc) ;
-    for (i=0 ; i<nb_sed ; i++) {
-        sed_loc_x[i+2] = sed_x[i] ;
-        sed_loc_y[i+2] = sed_y[i] ;
+    sed_loc = cpl_bivector_new(nb_sed + 4);
+    sed_loc_x = cpl_bivector_get_x_data(sed_loc);
+    sed_loc_y = cpl_bivector_get_y_data(sed_loc);
+    for (i=0; i<nb_sed; i++) {
+        sed_loc_x[i+2] = sed_x[i];
+        sed_loc_y[i+2] = sed_y[i];
     }
 
     /* Low bound */
-    sed_loc_x[1] = sed_loc_x[2] - wstep ;
+    sed_loc_x[1] = sed_loc_x[2] - wstep;
     if (sed_loc_x[2] < wmin) {
-        sed_loc_x[0] = sed_loc_x[1] - wstep ;
+        sed_loc_x[0] = sed_loc_x[1] - wstep;
     } else {
-        sed_loc_x[0] = wmin - wstep ;
+        sed_loc_x[0] = wmin - wstep;
     }
-    sed_loc_y[0] = 1e-20 ;
-    sed_loc_y[1] = 1e-20 ;
+    sed_loc_y[0] = 1e-20;
+    sed_loc_y[1] = 1e-20;
 
     /* High bound */
-    sed_loc_x[nb_sed+2] = sed_loc_x[nb_sed+1] + wstep ;
+    sed_loc_x[nb_sed+2] = sed_loc_x[nb_sed+1] + wstep;
     if (sed_loc_x[nb_sed+1] > wmax) {
-        sed_loc_x[nb_sed+3] = sed_loc_x[nb_sed+2] + wstep ;
+        sed_loc_x[nb_sed+3] = sed_loc_x[nb_sed+2] + wstep;
     } else {
-        sed_loc_x[nb_sed+3] = wmax + wstep ;
+        sed_loc_x[nb_sed+3] = wmax + wstep;
     }
-    sed_loc_y[nb_sed+2] = 1e-20 ;
-    sed_loc_y[nb_sed+3] = 1e-20 ;
+    sed_loc_y[nb_sed+2] = 1e-20;
+    sed_loc_y[nb_sed+3] = 1e-20;
 
     /* Create the output bivector */
-    out = cpl_vector_duplicate(waves) ;
-    out_biv = cpl_bivector_wrap_vectors((cpl_vector*)waves, out) ;
-
+    out = cpl_vector_duplicate(waves);
+    IRPLIB_DIAG_PRAGMA_PUSH_IGN(-Wcast-qual);
+    /* the X entry (waves) is not modified by cpl_bivector_interpolate_linear */
+    out_biv = cpl_bivector_wrap_vectors((cpl_vector*)waves, out);
+    IRPLIB_DIAG_PRAGMA_POP;
     /* Interpolate */
     if (cpl_bivector_interpolate_linear(out_biv, sed_loc) != CPL_ERROR_NONE) {
-        cpl_msg_error(cpl_func, "Cannot interpolate the wavelength") ;
-        cpl_bivector_unwrap_vectors(out_biv) ;
-        cpl_vector_delete(out) ;
-        cpl_bivector_delete(sed_loc) ;
-        return NULL ;
+        cpl_msg_error(cpl_func, "Cannot interpolate the wavelength");
+        cpl_bivector_unwrap_vectors(out_biv);
+        cpl_vector_delete(out);
+        cpl_bivector_delete(sed_loc);
+        return NULL;
     }
-    cpl_bivector_unwrap_vectors(out_biv) ;
-    cpl_bivector_delete(sed_loc) ;
+    cpl_bivector_unwrap_vectors(out_biv);
+    cpl_bivector_delete(sed_loc);
 
     /* Compute f0_jan */
-    f0_jan = 5513.15 / ( pow(cent_wl,3) * (exp(1.2848/cent_wl)-1) ) ;
+    f0_jan = 5513.15 / ( pow(cent_wl,3) * (exp(1.2848/cent_wl)-1) );
 
     /* Convert f0 Jansky -> ergs/s/cm^2/Angstrom */
-    f0_erg = f0_jan * 1e-26 * 1e7 * 3e18 / (1e4 * cent_wl*cent_wl*1e4*1e4) ;
+    f0_erg = f0_jan * 1e-26 * 1e7 * 3e18 / (1e4 * cent_wl*cent_wl*1e4*1e4);
 
     /* Scale out so that the central value is f0 */
-    cent_val = cpl_vector_get(out, cpl_vector_get_size(out)/2) ;
+    cent_val = cpl_vector_get(out, cpl_vector_get_size(out)/2);
     if (cent_val <= 0.0) {
-        cpl_msg_error(cpl_func, "Negative or 0 central value") ;
-        cpl_vector_delete(out) ;
-        return NULL ;
+        cpl_msg_error(cpl_func, "Negative or 0 central value");
+        cpl_vector_delete(out);
+        return NULL;
     }
-    cpl_vector_multiply_scalar(out, f0_erg/cent_val) ;
+    cpl_vector_multiply_scalar(out, f0_erg/cent_val);
 
     /* Return */
-    return out ;
+    return out;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -763,61 +777,61 @@ cpl_bivector * irplib_stdstar_get_sed(
         const char  *   seds_file,
         const char  *   sptype)
 {
-    cpl_table           *   seds ;
-    cpl_bivector        *   out ;
-    cpl_vector          *   wave ;
-    cpl_vector          *   sed ;
-    cpl_bivector        *   tmp ;
-    int                     nlines ;
+    cpl_table           *   seds;
+    cpl_bivector        *   out;
+    cpl_vector          *   wave;
+    cpl_vector          *   sed;
+    cpl_bivector        *   tmp;
+    int                     nlines;
 
     /* Test entries */
-    if (seds_file == NULL) return NULL ;
-    if (sptype == NULL) return NULL ;
+    if (seds_file == NULL) return NULL;
+    if (sptype == NULL) return NULL;
 
     /* Load the table */
     if ((seds = cpl_table_load(seds_file, 1, 0)) == NULL) {
-        cpl_msg_error(cpl_func, "Cannot load the table") ;
-        return NULL ;
+        cpl_msg_error(cpl_func, "Cannot load the table");
+        return NULL;
     }
 
     /* Check if the column is there */
     if (!cpl_table_has_column(seds, sptype)) {
-        cpl_msg_error(cpl_func, "SED of the requested star not available") ;
-        cpl_table_delete(seds) ;
-        return NULL ;
+        cpl_msg_error(cpl_func, "SED of the requested star not available");
+        cpl_table_delete(seds);
+        return NULL;
     }
 
     /* Get the nb lines */
-    nlines = cpl_table_get_nrow(seds) ;
+    nlines = cpl_table_get_nrow(seds);
 
     /* Get the wavelength as a vector */
     if ((wave = cpl_vector_wrap(nlines,
             cpl_table_get_data_double(seds, "Wavelength"))) == NULL) {
-        cpl_msg_error(cpl_func, "Cannot get the Wavelength column") ;
-        cpl_table_delete(seds) ;
-        return NULL ;
+        cpl_msg_error(cpl_func, "Cannot get the Wavelength column");
+        cpl_table_delete(seds);
+        return NULL;
     }
 
     /* Get the SED as a vector */
     if ((sed = cpl_vector_wrap(nlines,
             cpl_table_get_data_double(seds, sptype))) == NULL) {
-        cpl_msg_error(cpl_func, "Cannot get the SED column") ;
-        cpl_table_delete(seds) ;
-        cpl_vector_unwrap(wave) ;
-        return NULL ;
+        cpl_msg_error(cpl_func, "Cannot get the SED column");
+        cpl_table_delete(seds);
+        cpl_vector_unwrap(wave);
+        return NULL;
     }
-    tmp = cpl_bivector_wrap_vectors(wave, sed) ;
+    tmp = cpl_bivector_wrap_vectors(wave, sed);
 
     /* Create the output bivector */
-    out = cpl_bivector_duplicate(tmp) ;
+    out = cpl_bivector_duplicate(tmp);
 
     /* Free */
-    cpl_bivector_unwrap_vectors(tmp) ;
-    cpl_vector_unwrap(wave) ;
-    cpl_vector_unwrap(sed) ;
-    cpl_table_delete(seds) ;
+    cpl_bivector_unwrap_vectors(tmp);
+    cpl_vector_unwrap(wave);
+    cpl_vector_unwrap(sed);
+    cpl_table_delete(seds);
 
     /* Return  */
-    return out ;
+    return out;
 }
 /**@}*/
