@@ -1670,7 +1670,7 @@ static int hawki_sci_postproc_save_image(casu_fits *outim,
     hawki_pfits_get_ndit(plist,&ndit);
     hawki_pfits_get_dit(plist,&dit);
     texp = (float)(ndit)*dit;
-    cpl_propertylist_update_double(plist,"CASUEXPT",(double)texp);
+    cpl_propertylist_update_double(plist,"EFF_EXPT",(double)texp);
     texp = (float)(n*ndit)*dit;
     cpl_propertylist_update_double(plist,"EXPTIME",(double)texp);
     cpl_propertylist_update_double(plist,"TEXPTIME",texp);
@@ -1841,7 +1841,7 @@ static int hawki_sci_postproc_save_cat(casu_tfits *outcat,
         hawki_pfits_get_ndit(plist,&ndit);
         hawki_pfits_get_dit(plist,&dit);
         texp = (float)(ndit)*dit;
-        cpl_propertylist_update_double(plist,"CASUEXPT",(double)texp);
+        cpl_propertylist_update_double(plist,"EFF_EXPT",(double)texp);
         texp = (float)(n*ndit)*dit;
         cpl_propertylist_update_double(plist,"EXPTIME",(double)texp);
         cpl_propertylist_update_double(plist,"TEXPTIME",texp);
